package Ocsinventory::Agent::Backend::OS::Generic::Packaging::RPM;

use strict;
use warnings;
use Date::Parse;

# helper for MySQL date format
sub formatMySQL {
	my $param  = shift;
	my $result = "";
	if ( length($param) == 1 ) {
		$result = "0" . $param;
	}
	else {
		$result = $param;
	}
	return $result;
}

sub check {
  return unless can_run("rpm");

  # Some time rpm is a wrapper or an alias for another
  `rpm --version 2>&1`;
  return if ($? >> 8)!=0;
  1;
}

sub run {
  my $params = shift;
  my $common = $params->{common};
  my $logger = $params->{logger};

  my @list;
  my $buff;
  foreach (`rpm -qa --queryformat "%{NAME}.%{ARCH} %{VERSION}-%{RELEASE} --%{INSTALLTIME:date}-- --%{SIZE}-- %{SUMMARY}\n--\n" 2>/dev/null`) {
    if (! /^--/) {
      chomp;
      $buff .= $_;
    } elsif ($buff =~ s/^(\S+)\s+(\S+)\s+--(.*)--\s+--(.*)--\s+(.*)//) {

	( my $sec, my $min, my $hour, my $mday, my $mon, my $year ) = strptime($3);
	$year += 1900;
	my $softwareInstallDateMysql = $year . "-"
	  . formatMySQL( $mon + 1 ) . "-"
	  . formatMySQL($mday) . " "
	  . formatMySQL($hour) . ":"
	  . formatMySQL($min) . ":"
	  . formatMySQL($sec);

	$common->addSoftware({
		'NAME'          => $1,
		'VERSION'       => $2,
		'INSTALLDATE'   => $softwareInstallDateMysql,
		'FILESIZE'      => $4,
		'COMMENTS'      => $5,
		'FROM'          => 'rpm'
	});

    } else {
      $logger->debug("Should never go here!");
      $buff = '';
    }
  }
}

1;
