# The complet name of the package
# the path MUST be valid or the package won't be loaded
package Ocsinventory::Agent::Backend::OS::Linux::CernExtension;

use strict;
use Data::UUID;
use strict;
use warnings;
use Getopt::Long;
use Pod::Usage;

use Data::Dumper;


sub getLocationFromLdap{
    my $hostname = shift;
    my $ldapServer = 'xldap.cern.ch';
    my $ldapBinary = '/usr/bin/ldapsearch';
    my $reply = "";
    if ( !-x  $ldapBinary ){
	return $reply;
    }
    my $ldapCommand = "$ldapBinary -x -h $ldapServer -b dc=cern,dc=ch \"(&(objectClass=computer)(distinguishedName=CN=$hostname,OU=CERN Linux Computers,DC=cern,DC=ch))\" location";
    $reply = `$ldapCommand|grep "^location"`;
    chomp $reply;
    $reply =~ s/^location: //;
    return $reply;
}


# This is the check function. The agent runs it just once the module is loaded.
# If the function return false, the module and its children are not executed
# eg: OS::Linux and OS::Linux::* won't executed if this run() function return
# false
# It's a easy way to target a module just for a system like Linux in this case.
sub check { $^O =~ /^linux$/ }

# trim string whitespaces function
sub trim {
	my $string = shift;
	$string =~ s/^\s+//;
	$string =~ s/\s+$//;
	return $string;
}

# helper for MySQL date format
sub formatMySQL {
	my $param  = shift;
	my $result = "";
	if ( length($param) == 1 ) {
		$result = "0" . $param;
	}
	else {
		$result = $param;
	}
	return $result;
}

# main function which is executed by OCS
sub run {
	my $params     = shift;                          # I fetch the parameter
	my $logger = $params->{logger};
	my $config = $params->{config};
	my $idFileName = $config->{etcdir}[0]."/id.conf";    # system id file

	my $systemId       = "";
	my $systemArch     = "";
	my $systemBootTime = "";


	# test if id is already generated
	if ( -e $idFileName ) {
		# load the id
		open( IDFILE, "$idFileName" ) || do {
			$logger->error("Could not open file $idFileName for writing: $!");
			return ;
		};
		$systemId = <IDFILE>;
		close(IDFILE);
	}

	# if id cannot be read or not generated then generate a new one
	if ( trim($systemId) eq "" ) {
		my $ug = new Data::UUID;
		$systemId = $ug->create_str();

		open( IDFILE, ">$idFileName" ) || do {
			$logger->error("Could not open file $idFileName for writing: $!");
                        return ;
                };
		print IDFILE ($systemId);
		close(IDFILE);
	}

	# get the architecture
	$systemArch = trim(`uname -m`);

	# get the boot time
	my $input = "/proc/uptime";
	open( INPUT, "$input" ) || do {
		$logger->error("Could not open file $input: $!\n");
		return ;
	};
	my $secsago;
	while (<INPUT>) {
		($secsago) = ( split(/\s+/) )[0];
	}
	my $boottime = time() - $secsago;

	( my $sec, my $min, my $hour, my $mday, my $mon, my $year ) =
	  localtime($boottime);
	$year += 1900;
	$systemBootTime = $year . "-"
	  . formatMySQL( $mon + 1 ) . "-"
	  . formatMySQL($mday) . " "
	  . formatMySQL($hour) . ":"
	  . formatMySQL($min) . ":"
	  . formatMySQL($sec);

	# I need to get the common object to update inventory
	my $common = $params->{common};

	my $location = "";
	my $hostname = `/bin/hostname`;
	$hostname =~ s/.cern.ch//;
	if ( $hostname ){
	  $location = getLocationFromLdap($hostname);
	}

	# and I update the information collected
	$common->setCernExtension(
		{
			OS_ID        => $systemId,
			ARCHITECTURE => $systemArch,
			BOOT_TIME    => $systemBootTime,
			LOCATION     => $location
		}
	);
}

1;
